#include "X3D_PointLight.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

PointLight::PointLight()
  : X3DLightNode(),
    _attenuation(SFVec3f(1.0f,0.0f,0.0f)),
    _location   (SFVec3f(0.0f,0.0f,0.0f)),
    _radius     (100.0f)
{
  defineTypeName("PointLight", "Lighting");
}

PointLight::PointLight(const SFFloat       &ambientIntensity,
                       const SFVec3f &attenuation,
                       const SFColor &color,
                       const SFFloat       &intensity,
                       const SFVec3f &location,
                       const SFBool        &on,
                       const SFFloat       &radius)
  : X3DLightNode(ambientIntensity,color,intensity,on),
    _attenuation(attenuation),
    _location   (location),
    _radius     (radius)
{
  defineTypeName("PointLight", "Lighting");
}

PointLight::PointLight(const PointLight &N)
  : X3DLightNode(N),
    _attenuation(N._attenuation),
    _location   (N._location),
    _radius     (N._radius)
{
}

SFAbstractNode PointLight::clone() const
{
  return new PointLight(*this);
}

PointLight::~PointLight()
{
}

void PointLight::setAttenuation(const SFVec3f &attenuation)
{
  _attenuation = attenuation;
}

void PointLight::setLocation(const SFVec3f &location)
{
  _location = location;
}

void PointLight::setRadius(const SFFloat &radius)
{
  _radius = radius;
}

void PointLight::load(const X3DFileElement *element)
{
  X3DLightNode::load(element);

  int index;
  index = element->getIndexAttribute("attenuation");
  if (index != -1)
    _attenuation = SFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("location");
  if (index != -1)
    _location = SFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("radius");
  if (index != -1)
    _radius = element->getAttribute(index).toFloat();
}

SFString &PointLight::write(SFString &output) const
{
  X3DLightNode::write(output);
  
  if (_attenuation != SFVec3f(1.0f,0.0f,0.0f))  
    output += " attenuation=\"" + toSFString(_attenuation) + "\"";
  if (_location != SFVec3f(0.0f,0.0f,0.0f))  
    output += " location=\"" + toSFString(_location) + "\"";
  if (_radius != 100.0f)  
    output += " radius=\"" + toSFString(_radius) + "\"";
    
  return output;  
}

}
}
