#include "X3D_NurbsGroup.h"
#include "X3D_NurbsSurface.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsGroup::NurbsGroup()
  : X3DGroupingNode(), X3DBoundedObject(),
    _tesselationScale(1.0f)
{
  defineTypeName("NurbsGroup", "NURBS");
}

NurbsGroup::NurbsGroup(const SFVec3f &bboxCenter,
                       const SFVec3f &bboxSize,
                       const SFFloat &tesselationScale)
  : X3DGroupingNode(), X3DBoundedObject(bboxCenter, bboxSize),
    _tesselationScale(tesselationScale)
{
  defineTypeName("NurbsGroup", "NURBS");
}

NurbsGroup::NurbsGroup(const NurbsGroup &N)
  : X3DGroupingNode(N), X3DBoundedObject(N),
    _tesselationScale(N._tesselationScale)
{
}

SFAbstractNode NurbsGroup::clone() const
{
  return new NurbsGroup(*this);
}

NurbsGroup::~NurbsGroup()
{
}

void NurbsGroup::setTesselationScale(const SFFloat &tesselationScale)
{
  _tesselationScale = tesselationScale;
}

bool NurbsGroup::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<NurbsSurface *>(N) != 0)
    return X3DGroupingNode::addChild(N);
  
  if (N != 0)
    cx3d << "X3D::NurbsGroup::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::NurbsGroup::addChild : cannot add a NULL node!" << endl;
    
  
  return false;    
}

bool NurbsGroup::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<NurbsSurface *>(N) != 0)
    return X3DGroupingNode::setChild(N);
  
  if (N != 0)
    cx3d << "X3D::NurbsGroup::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::NurbsGroup::setChild : cannot add a NULL node!" << endl;
   
  return false;    
}

void NurbsGroup::load(const X3DFileElement *element)
{
  X3DBoundedObject::load(element);

  int index;
  index = element->getIndexAttribute("tesselationScale");
  if (index != -1)
    _tesselationScale = element->getAttribute(index).toFloat();
}

SFString &NurbsGroup::write(SFString &output) const
{
  X3DBoundedObject::write(output);
  
  if (_tesselationScale != 1.0f)
    output += " tesselationScale=\"" + toSFString(_tesselationScale) + "\"";

  return output;
}

}
}
