#include "MESH_ConnectedComComputer.h"
#include "MESH_ConnectedComComputerCoreVisitor.h"
#include "MESH_ConnectedComComputerMeshCoreVisitor.h"
#include "MESH_ConnectedComComputerStateVariables.h"
#include "DFSGraphTraversal.h"
#include "Progress.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

ConnectedComComputer::ConnectedComComputer()
: X3DMultiPassProcessor()
{
  progress = new Progress(256);
  stateVariables = Singleton<ConnectedComComputerStateVariables>::getInstance();
  stateVariables->setProgress(progress);
  
  graphTraversalMap["Vertex"] = new DFSGraphTraversal();
  graphTraversalMap["Vertex"]->setComponentVisitor(new ConnectedComComputerCoreVisitor(progress));
  
  graphTraversalMap["Mesh"] = new DFSGraphTraversal();
  graphTraversalMap["Mesh"]->setComponentVisitor(new ConnectedComComputerMeshCoreVisitor());
}

ConnectedComComputer::~ConnectedComComputer()
{
  Singleton<ConnectedComComputerStateVariables>::removeInstance();
  delete progress;
  delete graphTraversalMap["Vertex"];
  delete graphTraversalMap["Mesh"];
}

void ConnectedComComputer::compute(Scene *S)
{
  compute(static_cast<X3DNode *>(S));
  
  S->getProcessHistory().addEntry("MESH::ConnectedComComputer");
}

void ConnectedComComputer::compute(X3DNode *N)
{
  stateVariables->init();
  graphTraversalMap["Vertex"]->traverse(N);
  stateVariables->finish(); 

  stateVariables->init();
  graphTraversalMap["Mesh"]->traverse(N);
  stateVariables->finish(); 
}

void ConnectedComComputer::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

}
}  
