#include "X3D_WorldCoordTranslator.h"
#include "X3D_WorldCoordTranslatorMulCoreVisitor.h"
#include "X3D_WorldCoordTranslatorMulGroupingVisitor.h"
#include "X3D_WorldCoordTranslatorMulRenderingVisitor.h"
#include "X3D_WorldCoordTranslatorMulNavigationVisitor.h"
#include "X3D_WorldCoordTranslatorShareCoreVisitor.h"
#include "X3D_WorldCoordTranslatorReplaceGroupingVisitor.h"
#include "DFSGraphTraversal.h"
#include "X3D_X3DNode.h"
#include "X3D_Scene.h"

namespace X3DTK {
namespace X3D {

WorldCoordTranslator::WorldCoordTranslator()
{
  stateVariables = Singleton<WorldCoordTranslatorStateVariables>::getInstance();
    
  graphTraversalMap["Share"] = new DFSGraphTraversal();
  graphTraversalMap["Share"]->setComponentVisitor(new WorldCoordTranslatorShareCoreVisitor());

  graphTraversalMap["Mul"] = new DFSGraphTraversal();
//  graphTraversalMap["Mul"]->setComponentVisitor(new WorldCoordTranslatorMulCoreVisitor());
  graphTraversalMap["Mul"]->setComponentVisitor(new WorldCoordTranslatorMulGroupingVisitor());
  graphTraversalMap["Mul"]->setComponentVisitor(new WorldCoordTranslatorMulRenderingVisitor());
//  graphTraversalMap["Mul"]->setComponentVisitor(new WorldCoordTranslatorMulNavigationVisitor());
    
  graphTraversalMap["Replace"] = new DFSGraphTraversal();
  graphTraversalMap["Replace"]->setComponentVisitor(new WorldCoordTranslatorReplaceGroupingVisitor());
}

WorldCoordTranslator::~WorldCoordTranslator()
{
  Singleton<WorldCoordTranslatorStateVariables>::removeInstance();

  delete graphTraversalMap["Share"];
}

void WorldCoordTranslator::translate(Scene *S)
{
  translate(static_cast<X3DNode *>(S));
  
  S->getProcessHistory().addEntry("X3D::WorldCoordTranslator");
}

void WorldCoordTranslator::translate(SFNode N)
{
  stateVariables->init();
  graphTraversalMap["Share"]->traverse(N);
  stateVariables->finish();
  stateVariables->init();
  graphTraversalMap["Mul"]->traverse(N);
  stateVariables->finish();
  stateVariables->init();
  graphTraversalMap["Replace"]->traverse(N);
  stateVariables->finish();
}

}
}
